/******************************************************************************
* MSGLOOP(): demonstration of how to create modeless dialog boxes and keep    *
*            them alive from within a thread separate from the main thread.   *
*                             1998 by Mark Gamber                             *
*******************************************************************************
* By using this program or any part of it, the user assumes full respon-      *
* -sibility for it's use and may not hold the author liable for any loss or   *
* damage. If you cannot accept this restriction, the program may not be used  *
* and must be destroyed immediately.                                          *
******************************************************************************/

#include "windows.h"

// === Function Prototypes ====================================================

BOOL WINAPI MainDlgProc( HWND, UINT, WPARAM, LPARAM );
BOOL WINAPI GenericDlgProc( HWND, UINT, WPARAM, LPARAM );
DWORD ThreadProc( void );

// === Global Variables =======================================================

HINSTANCE hInst;                                //  Instance handle of this app
HANDLE hStopEvent;                   //  Event to tell all threads to shut down
int iThreadCount;                          //  Tracks number of running threads

// === Program Entry Point ====================================================

int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrev, LPSTR lpCmd,
					  int nShow )
{
	HWND hDlg;
	MSG msg;


	hInst = hInstance;                    //  First, create initial dialog box
	hDlg = CreateDialog( hInst, MAKEINTRESOURCE( 10000 ), NULL,
						 MainDlgProc );
	if( ! hDlg )
		return( FALSE );

	ShowWindow( hDlg, nShow );                         //  Show the dialog box
	UpdateWindow( hDlg );
	while( GetMessage( &msg, NULL, 0, 0 ) )        //  Start main message loop
	{
		if( IsDialogMessage( hDlg, &msg ) )
			continue;
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	return( FALSE );
}

// === Main Window (Dialog) Procedure =========================================

BOOL WINAPI MainDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
		case WM_INITDIALOG:
			iThreadCount = 0;                        //  No running threads yet
			                  //  Create event used to stop all running threads
			hStopEvent = CreateEvent( NULL, TRUE, FALSE, NULL );
			return( TRUE );


		case WM_COMMAND:
			if( wParam == IDCANCEL )           //  When dialog box is closed...
			{
				SetEvent( hStopEvent );          //  Signal threads to exit now
				while( iThreadCount )     //  Then wait for the thread count to
					Sleep( 250 );                // drop to zero (none running)
				CloseHandle( hStopEvent );          //  Kill the event and exit
				DestroyWindow( hDlg );
				PostQuitMessage( 0 );
				break;
			}
			if( wParam == 100 )                         //  Create a new thread
			{
				HANDLE hThread;
				DWORD dwID;
                                                           //  Start the thread
				hThread = CreateThread( NULL, 0,
										(LPTHREAD_START_ROUTINE)ThreadProc,
										NULL, 0, &dwID );
				if( hThread )                 //  If created, close it's handle
					CloseHandle( hThread );        //  Since it's not used here
			}
			break;
	}
	return( FALSE );
}                                                      //  End of MainDlgProc()

// === Thread Procedure =======================================================

DWORD ThreadProc( void )
{
	HWND hDlg;
	MSG msg;
	char szStr[ 32 ];
	int iCounter;

                  //  Create a modeless dialog box run control from this thread
	hDlg = CreateDialog( hInst, MAKEINTRESOURCE( 10001 ), NULL,
						 GenericDlgProc );
	if( ! hDlg )                            //  If not created, exit thread now
		return( 0 );
	++iThreadCount;                             //  Bump up thread count by one

	ShowWindow( hDlg, SW_SHOW );                    //  Show the new dialog box
	UpdateWindow( hDlg );

	iCounter = 0;                                 //  Initialize thread counter
	while( IsWindow( hDlg ) &&  //  While the dialog exists and no event signal
		   WaitForSingleObject( hStopEvent, 0 ) != WAIT_OBJECT_0 )
	{                                              //  Any messages to process?
		if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
		{                                  //  If so, do the message loop thing
			GetMessage( &msg, NULL, 0, 0 );
			if( ! IsDialogMessage( hDlg, &msg ) )
			{
				TranslateMessage( &msg );
				DispatchMessage( &msg );
			}
		}

		++iCounter;                   //  Increment internal thread counter and
		wsprintf( szStr, "%d", iCounter );       //  display so we can see this
		SetDlgItemText( hDlg, 100, szStr );       //  thread is doing something
	}

	--iThreadCount;                //  Before leaving, decrement global counter
	return( 0 );
}                                                       //  End of ThreadProc()

// === Dialog box run by ThreadProc() =========================================

BOOL WINAPI GenericDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
		case WM_INITDIALOG:
			return( TRUE );


		case WM_COMMAND:
			if( wParam == IDCANCEL )
			{
				DestroyWindow( hDlg );
				PostQuitMessage( 0 );
				break;
			}
			break;
	}
	return( FALSE );
}

// ============================================================================



